/*
 * Copyright (C) 2023-2024
 * Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package config

import (
	"os"
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestLoadConfig(t *testing.T) {
	t.Run("should load config from file", func(t *testing.T) {
		config := LoadConfig()

		expectedConfig := Config{}
		expectedConfig.Grpc.Server.Mock = false
		expectedConfig.Http.Server.Port = 8082
		expectedConfig.Grpc.Server.Port = 50052
		expectedConfig.Grpc.Router.Port = 50051
		expectedConfig.Logging.Level = "DEBUG"

		assert.Equal(t, expectedConfig, config)
	})

	t.Run("should load config from env", func(t *testing.T) {
		envVars := map[string]string{
			"HTTP_SERVER_PORT": "9090",
			"GRPC_SERVER_PORT": "1234",
			"GRPC_ROUTER_PORT": "5678",
			"LOGGING_LEVEL":    "ERROR",
		}

		for key, value := range envVars {
			assert.NoError(t, os.Setenv(key, value), "Setenv "+key+" should not return an error")
		}

		config := LoadConfig("testdata/empty_test_config.yml")

		for key := range envVars {
			assert.NoError(t, os.Unsetenv(key), "Unsetenv "+key+" should not return an error")
		}

		expectedConfig := Config{}
		expectedConfig.Grpc.Server.Mock = false
		expectedConfig.Http.Server.Port = 9090
		expectedConfig.Grpc.Server.Port = 1234
		expectedConfig.Grpc.Router.Port = 5678
		expectedConfig.Logging.Level = "ERROR"

		assert.Equal(t, expectedConfig, config)
	})

	t.Run("should overwrite config with env", func(t *testing.T) {
		envVars := map[string]string{
			"HTTP_SERVER_PORT": "9090",
			"GRPC_SERVER_PORT": "1234",
		}

		for key, value := range envVars {
			assert.NoError(t, os.Setenv(key, value), "Setenv "+key+" should not return an error")
		}

		config := LoadConfig()

		for key := range envVars {
			assert.NoError(t, os.Unsetenv(key), "Unsetenv "+key+" should not return an error")
		}

		expectedConfig := Config{}
		expectedConfig.Grpc.Server.Mock = false
		expectedConfig.Http.Server.Port = 9090
		expectedConfig.Grpc.Server.Port = 1234
		expectedConfig.Grpc.Router.Port = 50051
		expectedConfig.Logging.Level = "DEBUG"

		assert.Equal(t, expectedConfig, config)
	})
}
